#include <array>
#include "testlib.h"

using std::vector;

const int MAX_N_1 = 100;
const int MAX_N_2 = 1'000;
const int MAX_N_5 = 40'000;
const int MAX_N   = 100'000;

const int MAX_H_2 = 10;
const int MAX_H = 1'000'000'000;

void check_h(int h, int i) {
    ensuref(1 <= h && h <= MAX_H, "h_%d violates range [1; %d]", i, MAX_H);
}

void check_h_2(int h, int i) {
    ensuref(1 <= h && h <= MAX_H_2, "h_%d violates range [1; %d]", i, MAX_H_2);
}

void check_h_3(int h, int i) {
    static int prev = MAX_H + 1;
    static bool descend = true;
    ensuref(1 <= h && h <= MAX_H, "h_%d violates range [1; %d]", i, MAX_H);

    if (prev < h && descend) {
        descend = false;
    }
    if (!descend) {
        ensuref(prev <= h, "h_%d is lesser than previous value(%d < %d)", i, h, prev);
    }
    prev = h;
}

using check_h_t = void(*)(int h, int i);

struct group_policy {
    int max_n;
    check_h_t test_h;
};

std::array<group_policy, 8> policy{{
        { MAX_N, check_h },           // Samples
        { MAX_N_1, check_h },  // n <= 100
        { MAX_N_2, check_h },  // n <= 1000
        { MAX_N, check_h_2 },         // h <= 10
        { MAX_N, check_h_3 },         // bitonic
        { MAX_N, check_h },           // k = 1
        { MAX_N_5, check_h },  // n <= 4e4
        { MAX_N, check_h }            // n <= 1e5
}};

const int GROUP_ALL_PREFIX = 5;

void check_input(group_policy p, int group) {
    int n = inf.readInt(2, p.max_n, "n");
    inf.readEoln();

    auto h = inf.readInts(n, 1, MAX_H, "h");
    for (int i = 0; i < n; i++) {
        p.test_h(h[i], i);
    }
    inf.readEoln();

    vector<int> k = inf.readInts(n - 1, 1, n, "k");
    inf.readEoln();
    for (int i = 0; i + 1 < n; i++) {
        ensuref(group != GROUP_ALL_PREFIX || k[i] == 1,
                "k_%d = %d, violates group %d requirements\n", i + 1, k[i], group);
        ensuref(k[i] <= n - i - 1, "on union %d no element %d\n", i, k[i]);
    }

    inf.readEof();
}

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);
    auto group = validator.group();
    if (group.empty()) {
        group = "0";
    }
    int group_num = std::stoi(group);

    check_input(policy[group_num], group_num);

    return 0;
}