#include <bits/stdc++.h>

using namespace std;

using ll = long long;
using ull = unsigned long long;
using ld = long double;

#define pbc push_back
#define pob pop_back()
#define mp make_pair
#define all(a) (a).begin(), (a).end()
#define vin(a)        \
    for (auto& i : a) \
    cin >> i

// mt19937 rnd(chrono::steady_clock::now().time_since_epoch().count());

template <typename T1, typename T2>
inline void chkmin(T1& x, const T2& y) {
    if (y < x)
        x = y;
}

template <typename T1, typename T2>
inline void chkmax(T1& x, const T2& y) {
    if (x < y)
        x = y;
}

struct Fenwick {
    int n;
    vector<int> tree;
    Fenwick() = default;
    Fenwick(int n_) : n(n_), tree(n, 0) {
    }
    void upd(int pos, int val) {
        for (; pos < n; pos |= pos + 1) {
            tree[pos] += val;
        }
    }
    int get(int r) {
        int ans = 0;
        for (; r >= 0; r = (r & (r + 1)) - 1) {
            ans += tree[r];
        }
        return ans;
    }
};

struct Ids {
    int n;
    Fenwick st;
    Ids() = default;
    Ids(int n_) : n(n_), st(n) {
        for (int i = 0; i < n; ++i) {
            st.upd(i, 1);
        }
    }
    void Remove(int pos) {
        st.upd(pos, -1);
    }
    int GetPosById(int id) {
        int l = -1, r = n;
        while (l < r - 1) {
            int m = (l + r) / 2;
            if (st.get(m) >= id) {
                r = m;
            } else {
                l = m;
            }
        }
        return r;
    }
};

int n;
vector<int> h;
Ids ids;

struct Segment {
    vector<int> records_left;
    vector<int> records_right;
    ll total_hist;
    ll total_sum;
    int l, r;
    Segment() = default;
    Segment(int pos)
        : records_left({pos}),
          records_right({pos}),
          total_hist(h[pos]),
          total_sum(h[pos]),
          l(pos),
          r(pos) {
    }

    ll GetScore() const {
        // cerr << "l = " << l << " r = " << r << endl;
        // ll score = 0;
        // for (int i = l + 1; i < r; ++i) {
        //     int lp = *max_element(h.begin() + l, h.begin() + i + 1);
        //     int rp = *max_element(h.begin() + i, h.begin() + r + 1);
        //     score += min(lp, rp) - h[i];
        // }
        // return score;
        return total_hist - total_sum;
    }

    int GetMax() const {
        assert(h[records_left.front()] == h[records_right.front()]);
        return h[records_left.front()];
    }

    void Print() const {
        cerr << "l = " << l << " r = " << r << " total_hist = " << total_hist
             << " total_sum = " << total_sum << endl;
        cerr << "records_left = " << endl;
        for (auto i : records_left) {
            cerr << i << " ";
        }
        cerr << endl;
        cerr << "records_right = " << endl;
        for (auto i : records_right) {
            cerr << i << " ";
        }
        cerr << endl;
    }
};

vector<Segment> segments;

vector<int> MergeStacks(vector<int> lhs, vector<int> rhs) {
    while (!rhs.empty() && h[rhs.back()] <= h[lhs.front()]) {
        rhs.pop_back();
    }
    for (auto i : lhs) {
        rhs.push_back(i);
    }
    return rhs;
}

Segment Merge(Segment lhs, Segment rhs) {
    // cerr << "lhs = " << endl;
    // lhs.Print();
    // cerr << "rhs = " << endl;
    // rhs.Print();

    Segment ans;
    ans.l = lhs.l;
    assert(lhs.r + 1 == rhs.l);
    ans.r = rhs.r;
    ans.total_sum = lhs.total_sum + rhs.total_sum;
    ans.total_hist = lhs.total_hist + rhs.total_hist;

    int min_max = min(lhs.GetMax(), rhs.GetMax());

    // update left score
    {
        for (int i = (int)rhs.records_left.size() - 1; i > 0 && h[rhs.records_left[i]] < min_max;
             --i) {
            int len = rhs.records_left[i - 1] - rhs.records_left[i];
            assert(len > 0);
            ans.total_hist -= (ll)len * h[rhs.records_left[i]];
            ans.total_hist += (ll)len * min_max;
        }
    }
    // update right score
    {
        for (int i = (int)lhs.records_right.size() - 1; i > 0 && h[lhs.records_right[i]] < min_max;
             --i) {
            int len = lhs.records_right[i] - lhs.records_right[i - 1];
            assert(len > 0);
            ans.total_hist -= (ll)len * h[lhs.records_right[i]];
            ans.total_hist += (ll)len * min_max;
        }
    }

    ans.records_left = MergeStacks(std::move(lhs.records_left), std::move(rhs.records_left));
    ans.records_right = MergeStacks(std::move(rhs.records_right), std::move(lhs.records_right));

    // cerr << "ans = " << endl;
    // ans.Print();
    return ans;
}

signed main() {
    ios_base::sync_with_stdio(0), cin.tie(0), cout.tie(0);
    cout.precision(20), cout.setf(ios::fixed);
    cin >> n;
    h.resize(n);
    vin(h);
    ids = Ids(n);
    segments.resize(n);
    for (int i = 0; i < n; ++i) {
        segments[i] = Segment(i);
    }
    for (int query = 0; query < n - 1; ++query) {
        int k;
        cin >> k;
        int left_id = ids.GetPosById(k);
        int right_id = ids.GetPosById(k + 1);
        // cerr << "left_id = " << left_id << " right_id = " << right_id << endl;
        ids.Remove(right_id);
        Segment result = Merge(std::move(segments[left_id]), std::move(segments[right_id]));
        segments[left_id] = std::move(result);
        ll score = segments[left_id].GetScore();
        cout << score << "\n";
    }
    return 0;
}
