#include <bits/stdc++.h>

/*
 ! WARNING: MOD must be prime.
 ! WARNING: MOD must be less than 2^30.
 * Use .get() to get the stored value.
 */

template <uint32_t mod>
class montgomery {
    static_assert(mod < uint32_t(1) << 30, "mod < 2^30");
    using mint = montgomery<mod>;

   private:
    uint32_t value;

    static constexpr uint32_t inv_neg_mod = []() {
        uint32_t x = mod;
        for (int i = 0; i < 4; ++i) {
            x *= uint32_t(2) - mod * x;
        }
        return -x;
    }();
    static_assert(mod * inv_neg_mod == -1);

    static constexpr uint32_t neg_mod = (-uint64_t(mod)) % mod;

    static uint32_t reduce(const uint64_t& value) {
        return (value + uint64_t(uint32_t(value) * inv_neg_mod) * mod) >> 32;
    }

    inline static const mint ONE = mint(1);

   public:
    montgomery() : value(0) {}
    montgomery(const mint& x) : value(x.value) {}

    template <typename T,
              typename U = std::enable_if_t<std::is_integral<T>::value>>
    montgomery(const T& x)
        : value(
              !x ? 0
                 : reduce(int64_t(x % int32_t(mod) + int32_t(mod)) * neg_mod)) {
    }

    static constexpr uint32_t get_mod() { return mod; }

    uint32_t get() const {
        auto real_value = reduce(value);
        return real_value < mod ? real_value : real_value - mod;
    }

    template <typename T>
    mint power(T degree) const {
        degree =
            (degree % int32_t(mod - 1) + int32_t(mod - 1)) % int32_t(mod - 1);
        mint prod = 1, a = *this;
        for (; degree > 0; degree >>= 1, a *= a)
            if (degree & 1)
                prod *= a;

        return prod;
    }

    mint inv() const { return power(-1); }

    mint& operator=(const mint& x) {
        value = x.value;
        return *this;
    }

    mint& operator+=(const mint& x) {
        if (int32_t(value += x.value - (mod << 1)) < 0) {
            value += mod << 1;
        }
        return *this;
    }

    mint& operator-=(const mint& x) {
        if (int32_t(value -= x.value) < 0) {
            value += mod << 1;
        }
        return *this;
    }

    mint& operator*=(const mint& x) {
        value = reduce(uint64_t(value) * x.value);
        return *this;
    }

    mint& operator/=(const mint& x) { return *this *= x.inv(); }

    friend mint operator+(const mint& x, const mint& y) { return mint(x) += y; }

    friend mint operator-(const mint& x, const mint& y) { return mint(x) -= y; }

    friend mint operator*(const mint& x, const mint& y) { return mint(x) *= y; }

    friend mint operator/(const mint& x, const mint& y) { return mint(x) /= y; }

    mint& operator++() { return *this += ONE; }

    mint& operator--() { return *this -= ONE; }

    mint operator++(int) {
        mint prev = *this;
        *this += ONE;
        return prev;
    }

    mint operator--(int) {
        mint prev = *this;
        *this -= ONE;
        return prev;
    }

    mint operator-() const { return mint(0) - *this; }

    bool operator==(const mint& x) const { return get() == x.get(); }

    bool operator!=(const mint& x) const { return get() != x.get(); }

    bool operator<(const mint& x) const { return get() < x.get(); }

    template <typename T>
    explicit operator T() {
        return get();
    }

    friend std::istream& operator>>(std::istream& in, mint& x) {
        std::string s;
        in >> s;
        x = 0;
        bool neg = s[0] == '-';
        for (const auto c : s)
            if (c != '-')
                x = x * 10 + (c - '0');

        if (neg)
            x *= -1;

        return in;
    }

    friend std::ostream& operator<<(std::ostream& out, const mint& x) {
        return out << x.get();
    }
};

constexpr uint32_t MOD = 1'000'000'007;
// constexpr uint32_t MOD = 998'244'353;
using mint = montgomery<MOD>;
using ll = long long;

using namespace std;
using pi = pair<int, int>;
using vi = vector<int>;

mint solve(int n, int m, vector<pi> pts) {
    const int N = max(n, m);

    vector<vi> a(N, vi(N));
    vector<int> cnt(2*N-1);
    for (auto [x, y] : pts) {
        a[x-1][y-1] = 1;
    }
    for (int i = 0; i < N; ++i) {
        for (int j = 0; j < N; ++j) {
            cnt[i+j] += a[i][j];
        }
    }

    vector<vector<vector<mint>>> dp(2*N-1,
                                    vector<vector<mint>>(N, vector<mint>(N)));
    dp[1][0][1] = 2;
    for (int diag = 1; diag  < 2*N - 3; ++diag) {
        for (int x = max(0, diag-N+1); x < N && x <= diag; ++x) {
            for (int y = x + 1; y < N && y <= diag; ++y) {
                if (a[diag-x][x] + a[diag-y][y] != cnt[diag]) {
                    dp[diag][x][y] = 0;
                } else {
                    for (int dx = 0; dx <= 1; ++dx)
                    for (int dy = 0; dy <= 1; ++dy) {
                        if (x + dx < y + dy && y + dy < N) {
                            dp[diag+1][x+dx][y+dy] += dp[diag][x][y];
                        }
                    }
                }
            }
        }
    }

    const int d = n + m - 3;
    return dp[d][m-2][m-1];
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);

    int n, m, k;
    cin >> n >> m >> k;
    
    vector<pi> pts(k);
    for (auto& [x, y] : pts) {
        cin >> x >> y;
    }

    auto ans = solve(n, m, pts);
    cout << ans << '\n';
}